package test.CloudAppTest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import schedframe.events.scheduling.SchedulingEvent;
import schedframe.resources.ResourceStatus;
import schedframe.resources.computing.ComputingResource;
import schedframe.resources.computing.Core;
import schedframe.resources.computing.Node;
import schedframe.resources.computing.Processor;
import schedframe.resources.units.ProcessingElements;
import schedframe.resources.units.ResourceUnit;
import schedframe.resources.units.ResourceUnitName;
import schedframe.resources.units.StandardResourceUnitName;
import schedframe.scheduling.manager.resources.ClusterResourceManager;
import schedframe.scheduling.manager.resources.ResourceManager;
import schedframe.scheduling.manager.tasks.JobRegistry;
import schedframe.scheduling.plan.SchedulingPlanInterface;
import schedframe.scheduling.plan.impl.SchedulingPlan;
import schedframe.scheduling.plugin.ModuleList;
import schedframe.scheduling.queue.TaskQueue;
import schedframe.scheduling.queue.TaskQueueList;
import schedframe.scheduling.tasks.TaskInterface;
import test.Node_Fan_Mapping;
import example.localplugin.BaseLocalSchedulingPlugin;
import gridsim.dcworms.DCWormsTags;

public class CB1_FCFS_Random_SP_3TierApp extends BaseLocalSchedulingPlugin {

	private Random rand;
	
	public CB1_FCFS_Random_SP_3TierApp () {
		rand = new Random(47);
		Node_Fan_Mapping.init();
	}

	public SchedulingPlanInterface<?> schedule(SchedulingEvent event, TaskQueueList queues, JobRegistry jobRegistry,
			 ResourceManager resManager, ModuleList modules) {

		ClusterResourceManager resourceManager = (ClusterResourceManager) resManager;
		SchedulingPlan plan = new SchedulingPlan();
		// choose the events types to serve.
		// Different actions for different events are possible.
		switch (event.getType()) {
		case START_TASK_EXECUTION:
		case TASK_FINISHED:
		//case TIMER:
			// our tasks are placed only in first queue (see
			// BaseLocalSchedulingPlugin.placeJobsInQueues() method)
			TaskQueue q = queues.get(0);
			
			// check all tasks in queue
			for (int i = 0; i < q.size(); i++) {
				TaskInterface<?> task = q.get(i);
				// if status of the tasks in READY
				if (task.getStatus() == DCWormsTags.READY) {

					Map<ResourceUnitName, ResourceUnit> choosenResources = chooseResourcesForExecution(resourceManager, task);
					if (choosenResources != null) {
						addToSchedulingPlan(plan, task, choosenResources);
					}
				}
			}
			break;
		}
		return plan;
	}

	@SuppressWarnings("unchecked")
	private Map<ResourceUnitName, ResourceUnit> chooseResourcesForExecution(
			ClusterResourceManager resourceManager, TaskInterface<?> task) {

		Map<ResourceUnitName, ResourceUnit> map = new HashMap<ResourceUnitName, ResourceUnit>();
		
		List<Node> nodes = resourceManager.getNodes();
		List<Node> avNodes = filterNodes(nodes, task);
		if(avNodes.size() == 0)
			return null;
		Node node = chooseNode(avNodes, task);
		int cpuRequest;
		try {
			cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
		} catch (NoSuchFieldException e) {
			cpuRequest = 0;
		}

		if (cpuRequest != 0) {

			List<ComputingResource> choosenResources = new ArrayList<ComputingResource>();	
			
			List<Processor> processors = node.getProcessors();			
			for (int i = 0; i < processors.size() && cpuRequest > 0; i++) {
				if (processors.get(i).getStatus() == ResourceStatus.FREE) {
					choosenResources.add(processors.get(i));
					cpuRequest--;
				}
			}
			if (cpuRequest > 0) {
				return null;
			}
			//choosenResources.add(node.getProcessors().get(0));
			ProcessingElements pe = new ProcessingElements();
			pe.addAll(choosenResources);
			map.put(StandardResourceUnitName.PE, pe);
			return map;
		}
		return null;
	}
	
	private List<Node> filterNodes(List<Node> nodes, TaskInterface<?> task){
		List<Node> filteredNodes = new ArrayList<Node>();
		int cpuRequest;
		try {
			cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
		} catch (NoSuchFieldException e) {
			cpuRequest = 0;
		}
		for (Node node : nodes) {

			if (cpuRequest != 0) {

				List<Core> cores = node.getCores();
				if (cores.size() < cpuRequest) {
					if(cores.size() == 0){
						if(node.getProcessors().size() < cpuRequest)
							continue;
					}
				}

				int freeCores = 0;
				for(Core core: cores){
					if(core.getStatus() == ResourceStatus.FREE)
						freeCores++;
				}
				
				if(freeCores < cpuRequest)
					continue;
				
				filteredNodes.add(node);
			}
		}
		
		return filteredNodes;
	}
	
	private Node chooseNode(List<Node> nodes, TaskInterface<?> task){
		String appIdString = task.getJobId();
		int appId = Integer.valueOf(appIdString);
		int row = 0;//0 - random, 1 - front, 2 - back
		if(appId >= 1 && appId <= 5){
			row = 1;
		} else if (appId >= 6 && appId <= 10){
			row = 2;
		}
		List<Node> nodesF = new ArrayList<Node>();
		List<Node> nodesB = new ArrayList<Node>();
		for (Node node : nodes){
			String nodeName = node.getName().substring(node.getName().length()-2, node.getName().length());
			int nodeNumber =  Integer.valueOf(nodeName);
			/*int nodeRow = (nodeNumber <= 9) ? 1 : 2;
			if(nodeRow == row){
				return node;
			}*/
			
			if(nodeNumber <=9)
				nodesF.add(node);
			else if(nodeNumber <=18)
				nodesB.add(node);
		}
		if(row == 1)
			return nodesF.get(rand.nextInt(nodesF.size()));
		else if(row == 2)
			return nodesB.get(rand.nextInt(nodesB.size()));
		else
			return nodes.get(rand.nextInt(nodes.size()));
	}

}

